use std::fs;

use actix_multipart::form::{tempfile::TempFile, text::Text, MultipartForm};
use actix_web::{post, HttpResponse};
use lettre::{
    message::{header::ContentType, Attachment, MultiPart, SinglePart},
    Message,
};
use maud::{html, PreEscaped, DOCTYPE};
use tracing::{error, info};

#[derive(Debug, MultipartForm)]
struct MtForm {
    #[multipart(rename = "firstname")]
    first_name: Text<String>,
    #[multipart(rename = "lastname")]
    last_name: Text<String>,
    #[multipart(rename = "parentfirstname")]
    parent_first_name: Text<String>,
    #[multipart(rename = "parentlastname")]
    parent_last_name: Text<String>,
    birthdate: Text<String>,
    gender: Text<String>,
    street: Text<String>,
    city: Text<String>,
    state: Text<String>,
    zip: Text<i32>,
    cellphone: Text<String>,
    parentphone: Text<String>,
    email: Text<String>,
    parentemail: Text<String>,
    school: Text<String>,
    grade: Text<String>,
    #[multipart(rename = "pastorfirstname")]
    pastor_first_name: Text<String>,
    #[multipart(rename = "pastorlastname")]
    pastor_last_name: Text<String>,
    #[multipart(rename = "churchattendance")]
    church_attendance: Text<String>,
    #[multipart(rename = "tfcgroup")]
    tfc_group: Text<String>,
    shirt: Text<String>,
    trip: Text<String>,
    #[multipart(rename = "tripnotes")]
    trip_notes: Text<String>,
    #[multipart(rename = "relationship-with-jesus")]
    relationship_with_jesus: Text<String>,
    #[multipart(rename = "testimony")]
    testimony: Text<String>,
    #[multipart(rename = "involvement-with-group")]
    involvement_with_group: Text<String>,
    #[multipart(rename = "reasons-for-trip-choice")]
    reasons: Text<String>,
    strengths: Text<String>,
    weaknesses: Text<String>,
    #[multipart(rename = "previous-trip-info")]
    previous_trip_info: Text<String>,
    #[multipart(rename = "attitude-torward-work")]
    attitude: Text<String>,
    #[multipart(rename = "relevant-notes")]
    relevant_notes: Text<String>,
    #[multipart(rename = "final-agreement")]
    final_agreement: Text<String>,
    registration: Text<String>,
    #[multipart(rename = "image")]
    file: TempFile,
}

impl MtForm {
    async fn build_email(&self) -> PreEscaped<std::string::String> {
        html! {
            (DOCTYPE)
                meta charset="utf-8";
                html {
                    head {
                        title { (self.first_name.0) " " (self.last_name.0) " signed up for mission trip!" }
                        style {
                            "table { border-collapse: collapse; width: 100% }"
                                "td, th { padding: 8px }"
                                "td { text-align: left; width: 70%; word-wrap: break-word }"
                                "th { text-align: right; border-right: 1px solid #ddd }"
                                "tr { border-bottom: 1px solid #ddd }"
                                "h1 { text-align: center }"
                        }
                    }
                    body {
                        h1 { "Mission trip self for " (self.first_name.0) " " (self.last_name.0) "!" }
                        hr;
                        table {
                            tr {
                                th { "Name" }
                                td { (self.first_name.0) " " (self.last_name.0) }
                            }
                            tr {
                                th { "Parent" }
                                td { (self.parent_first_name.0) (self.parent_last_name.0) }
                            }
                            tr {
                                th { "Birthdate" }
                                td { (self.birthdate.0) }
                            }
                            tr {
                                th { "Gender" }
                                td { (self.gender.0) }
                            }
                            tr {
                                th { "Street" }
                                td { (self.street.0) }
                            }
                            tr {
                                th { "City" }
                                td { (self.city.0) }
                            }
                            tr {
                                th { "State" }
                                td { (self.state.0) }
                            }
                            tr {
                                th { "Zip" }
                                td { (self.zip.0) }
                            }
                            tr {
                                th { "Phone" }
                                td { (self.cellphone.0) }
                            }
                            tr {
                                th { "Parent Phone" }
                                td { (self.parentphone.0) }
                            }
                            tr {
                                th { "Email" }
                                td { (self.email.0) }
                            }
                            tr {
                                th { "Parent Email" }
                                td { (self.parentemail.0) }
                            }
                            tr {
                                th { "School" }
                                td { (self.school.0) }
                            }
                            tr {
                                th { "Grade" }
                                td { (self.grade.0) }
                            }
                            tr {
                                th { "Pastor" }
                                td { (self.pastor_first_name.0) (self.pastor_last_name.0) }
                            }
                            tr {
                                th { "Church Attendance" }
                                td { (self.church_attendance.0) }
                            }
                            tr {
                                th { "TFC Group" }
                                td { (self.tfc_group.0) }
                            }
                            tr {
                                th { "T-Shirt Size" }
                                td { (self.shirt.0) }
                            }
                            tr {
                                th { "Trip Choice" }
                                td { (self.trip.0) }
                            }
                            tr {
                                th { "Extra Trip Notes" }
                                td { (self.trip_notes.0) }
                            }
                            tr {
                                th { "Relationship with Jesus" }
                                td { (self.relationship_with_jesus.0) }
                            }
                            tr {
                                th { "Testimony" }
                                td { (self.testimony.0) }
                            }
                            tr {
                                th { "Involvement with TFC or Youth Group" }
                                td { (self.involvement_with_group.0) }
                            }
                            tr {
                                th { "Reasons for trip choice" }
                                td { (self.reasons.0) }
                            }
                            tr {
                                th { "Strengths" }
                                td { (self.strengths.0) }
                            }
                            tr {
                                th { "Weaknesses" }
                                td { (self.weaknesses.0) }
                            }
                            tr {
                                th { "Previous Trips" }
                                td { (self.previous_trip_info.0) }
                            }
                            tr {
                                th { "Attitude Torward Work" }
                                td { (self.attitude.0) }
                            }
                            tr {
                                th { "Other Relevant Info" }
                                td { (self.relevant_notes.0) }
                            }
                            tr {
                                th { "Final Agreement" }
                                td { (self.final_agreement.0) }
                            }
                            tr {
                                th { "Registration" }
                                td { (self.registration.0) }
                            }
                        }
                    }
                }
        }
    }
}

#[post("/api/mt-form")]
pub async fn mt_form(MultipartForm(form): MultipartForm<MtForm>) -> HttpResponse {
    let first = form.first_name.clone();
    let last = form.last_name.clone();
    let email_subject = format!("{} {} signed up for mission trip!", first, last);
    let filename_noext = format!("{}_{}", first, last);
    info!("{first} {last} signed up for mission trip!");
    let email = form.build_email().await;
    let mut path = String::from("");
    let mut file_exists = false;
    let mut filename = String::from("");
    info!("{:?}", file_exists);
    if let Some(file) = form.file.file_name {
        if let Some(ext) = file.rsplit('.').next() {
            filename = format!("{}.{}", filename_noext, ext);
            path = format!("./tmp/{}.{}", filename_noext, ext);
        } else {
            path = format!("./tmp/{}", file);
        }
        // let path = format!("./tmp/{}", file);
        info!("saving to {}", path);
        match form.file.file.persist(&path) {
            Ok(f) => {
                info!("{:?}", f);
                if f.metadata().unwrap().len() > 0 {
                    file_exists = true;
                }
            }
            Err(e) => info!("{:?}: Probably a missing image", e),
        }
    }

    let multi = if file_exists {
        let filebody = fs::read(path);
        let content_type = ContentType::parse("image/jpg").unwrap();
        let attachment = Attachment::new(filename).body(filebody.unwrap(), content_type);
        info!("{:?}", attachment);
        MultiPart::mixed()
            .singlepart(SinglePart::html(email.into_string()))
            .singlepart(attachment)
    } else {
        MultiPart::alternative_plain_html(String::from("Testing"), email.into_string())
    };

    if let Ok(m) = Message::builder()
        .from(
            "TFC ADMIN <no-reply@mail.tfcconnection.org>"
                .parse()
                .unwrap(),
        )
        .to("Chris Cochrun <chris@tfcconnection.org>".parse().unwrap())
        .subject(email_subject)
        .multipart(multi)
    {
        let future = crate::email::send_email(m);
        actix_rt::spawn(future);
    } else {
        error!("Email incorrect");
    }

    HttpResponse::Ok().body("thankyou")
}
